@@ -15,7 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Synapse admin API and regular Matrix API clients
+"""Synapse Admin API and regular Matrix API clients
 
 Most API calls defined in this module respect the API's defaults and only pass
 what's necessary in the request body.
@@ -24,7 +24,7 @@ A fully qualified Matrix user ID looks like this: @user:server, where server
 often is a domain name only, e.g @user@example.org
 
 See https://github.com/matrix-org/synapse/tree/master/docs/admin_api for
-documentation of the Synapse admin APIs and the Matrix spec at
+documentation of the Synapse Admin APIs and the Matrix spec at
 https://matrix.org/docs/spec/#matrix-apis.
 """
 
@@ -382,7 +382,7 @@ class Matrix(ApiRequest):
 
 
 class SynapseAdmin(ApiRequest):
-    """Synapse admin API client
+    """Synapse Admin API client
 
     Inheritance:
         ApiRequest (object): parent class containing general properties and
@@ -424,7 +424,7 @@ class SynapseAdmin(ApiRequest):
             _guests (bool): enable/disable fetching of guest users
             _deactivated (bool): enable/disable fetching of deactivated users
             _name (string): user name localpart to search for, see Synapse
-                admin API docs for details
+                Admin API docs for details
             _user_id (string): fully qualified Matrix user ID to search for
             _admin (bool or None): whether to filter for admins. a None
                 does not filter.
@@ -457,14 +457,14 @@ class SynapseAdmin(ApiRequest):
             _deactivated (bool): Enable/disable fetching of deactivated
                 users.
             _name (string): User name localpart to search for, see Synapse
-                admin API docs for details.
+                Admin API docs for details.
             _user_id (string): Fully qualified Matrix user ID to search for.
             _from (string): Offsets user list by this number, used for
                 pagination.
 
         Yields:
-            dict: The admin API response for listing accounts.
-                https://matrix-org.github.io/synapse/latest/admin_api/user_admin_api.html#list-accounts
+            dict: The Admin API response for listing accounts.
+                https://element-hq.github.io/synapse/latest/admin_api/user_admin_api.html#list-accounts
         """
         while _from is not None:
             response = self.user_list(_from, _limit, _guests, _deactivated,
@@ -483,8 +483,8 @@ class SynapseAdmin(ApiRequest):
                 passes as we need some Matrix API functionality here.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
         """
 
         rooms = self.query("get", "v1/users/{user_id}/joined_rooms",
@@ -503,11 +503,11 @@ class SynapseAdmin(ApiRequest):
         Args:
             user_id (string): fully qualified Matrix user ID
             gdpr_erase (bool): enable/disable gdpr-erasing the user, see
-                Synapse admin API docs for details.
+                Synapse Admin API docs for details.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
         """
         return self.query("post", "v1/deactivate/{user_id}", data={
             "erase": gdpr_erase
@@ -520,12 +520,12 @@ class SynapseAdmin(ApiRequest):
             user_id (string): fully qualified Matrix user ID
             password (string): new password that should be set
             no_logout (bool): the API defaults to logging out the user after
-                password reset via the admin API, this option can be used to
+                password reset via the Admin API, this option can be used to
                 disable this behaviour.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-            an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+            an exception occured. See Synapse Admin API docs for details.
         """
         data = {"new_password": password}
         if no_logout:
@@ -536,15 +536,15 @@ class SynapseAdmin(ApiRequest):
     def user_details(self, user_id):
         """Get information about a given user
 
-        Note that the admin API docs describe this function as "Query User
+        Note that the Admin API docs describe this function as "Query User
         Account".
 
         Args:
             user_id (string): fully qualified Matrix user ID
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
 
         """
         return self.query("get", "v2/users/{user_id}", user_id=user_id)
@@ -569,8 +569,8 @@ class SynapseAdmin(ApiRequest):
             _expire_ts (int):  token should expire after this date/time - a
                 unix timestamp in ms.
         Returns:
-            string: JSON string containing the admin API's response or None if
-            an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+            an exception occured. See Synapse Admin API docs for details.
         """
         expire_ts = None
         if expire_days:
@@ -603,7 +603,8 @@ class SynapseAdmin(ApiRequest):
                     avatar_url, admin, deactivation, user_type, lock):
         """ Create or update information about a given user
 
-        Threepid should be passed as a tuple in a tuple
+        The threepid argument must be passed as a tuple in a tuple (which is
+        what we usually get from a Click multi-arg option)
         """
         data = {}
         if password:
@@ -611,9 +612,12 @@ class SynapseAdmin(ApiRequest):
         if display_name:
             data.update({"displayname": display_name})
         if threepid:
-            data.update({"threepids": [
-                {"medium": k, "address": i} for k, i in dict(threepid).items()
-            ]})
+            if threepid == (('', ''),):  # empty strings clear all threepids
+                data.update({"threepids": []})
+            else:
+                data.update({"threepids": [
+                    {"medium": m, "address": a} for m, a in threepid
+                ]})
         if avatar_url:
             data.update({"avatar_url": avatar_url})
         if admin is not None:
@@ -642,8 +646,8 @@ class SynapseAdmin(ApiRequest):
             user_id (string): Fully qualified Matrix user ID.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
         """
         return self.query("get", "v2/users/{user_id}/devices",
                           user_id=user_id)
@@ -749,7 +753,7 @@ class SynapseAdmin(ApiRequest):
         of 3PID it is as medium.
         """
         return self.query("get", "v1/threepid/{medium}/users/{address}",
-                          address=address)
+                          address=address, medium=medium)
 
     def room_join(self, room_id_or_alias, user_id):
         """ Allow an administrator to join an user account with a given user_id
@@ -770,6 +774,34 @@ class SynapseAdmin(ApiRequest):
             "dir": "b" if reverse else None
         })
 
+    def room_list_paginate(self, limit, name, order_by, reverse, _from=0):
+        """ Yields API responses for room listing.
+
+        Args:
+            limit (int): Maximum number of rooms returned per pagination.
+            name (string or None): Search for a room by name. Passed as
+                `search_term` in the room list API. Use Python None to avoid
+                searching.
+            order_by (string): Synapse Room list API specific argument.
+            reverse (bool): Whether the results should be
+            _from (int): Initial offset in pagination.
+
+        Yields:
+            dict: The Admin API response for listing accounts.
+                https://element-hq.github.io/synapse/latest/admin_api/rooms.html#list-room-api
+        """
+        while _from is not None:
+            response = self.query("get", "v1/rooms", params={
+                "from": _from,
+                "limit": limit,
+                "search_term": name,
+                "order_by": order_by,
+                "dir": "b" if reverse else None
+            })
+            yield response
+            _from = response.get("next_batch", None)
+            self.log.debug(f"room_list_paginate: next from value = {_from}")
+
     def room_details(self, room_id):
         """ Get details about a room
         """
@@ -787,8 +819,8 @@ class SynapseAdmin(ApiRequest):
             room_id (string)
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
         """
         return self.query("get", "v1/rooms/{room_id}/state", room_id=room_id)
 
@@ -803,8 +835,8 @@ class SynapseAdmin(ApiRequest):
             room_id (string): If left out, all rooms are fetched.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
         """
         if room_id:
             # We use the "name search" possibility of the room list API to get
@@ -843,7 +875,7 @@ class SynapseAdmin(ApiRequest):
         return rooms
 
     def room_delete(self, room_id, new_room_user_id, room_name, message,
-                    block, no_purge):
+                    block, no_purge, force_purge):
         """ Delete a room and purge it if requested
         """
         data = {
@@ -857,11 +889,13 @@ class SynapseAdmin(ApiRequest):
             data.update({"room_name": room_name})
         if message:
             data.update({"message": message})
+        if force_purge:
+            data.update({"force_purge": force_purge})
         return self.query("delete", "v1/rooms/{room_id}", data=data,
                           room_id=room_id)
 
     def room_delete_v2(self, room_id, new_room_user_id, room_name, message,
-                       block, purge):
+                       block, purge, force_purge):
         """ Delete a room asynchronously and purge it if requested
         """
         data = {
@@ -875,6 +909,8 @@ class SynapseAdmin(ApiRequest):
             data.update({"room_name": room_name})
         if message:
             data.update({"message": message})
+        if force_purge:
+            data.update({"force_purge": force_purge})
         return self.query("delete", "v2/rooms/{room_id}", data=data,
                           room_id=room_id)
 
@@ -894,8 +930,8 @@ class SynapseAdmin(ApiRequest):
             block (boolean): Whether to block or unblock a room.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occurred. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occurred. See Synapse Admin API docs for details.
         """
         # TODO prevent usage on versions before 1.48
         data = {
@@ -911,8 +947,8 @@ class SynapseAdmin(ApiRequest):
             room_id (string): Fully qualified Matrix room ID.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
         """
         # TODO prevent usage on versions before 1.48
         return self.query("get", "v1/rooms/{room_id}/block", room_id=room_id)
@@ -995,8 +1031,8 @@ class SynapseAdmin(ApiRequest):
             server_name=server_name, media_id=media_id
         )
 
-    def media_delete_by_date_or_size(self, server_name, before_days, before,
-                                     _before_ts, _size_gt, delete_profiles):
+    def media_delete_by_date_or_size(self, before_days, before, _before_ts,
+                                     _size_gt, delete_profiles):
         """ Delete local media by date and/or size FIXME and/or?
         """
         if before_days:
@@ -1017,7 +1053,6 @@ class SynapseAdmin(ApiRequest):
         self.log.info("which is the date: %s",
                       self._datetime_from_timestamp(before_ts))
         params = {
-            "server_name": server_name,
         }
         if before_ts is not None:
             params.update({
@@ -1035,8 +1070,7 @@ class SynapseAdmin(ApiRequest):
                 "keep_profiles": "false"
             })
         return self.query(
-            "post", "v1/media/{server_name}/delete", data={}, params=params,
-            server_name=server_name
+            "post", "v1/media/delete", data={}, params=params
         )
 
     def media_protect(self, media_id):
@@ -1144,8 +1178,8 @@ class SynapseAdmin(ApiRequest):
                 timestamp.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
 
         """
         result = self.query("get", "v1/registration_tokens", params={
@@ -1177,8 +1211,8 @@ class SynapseAdmin(ApiRequest):
                 timestamp.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
 
         """
         result = self.query("get", "v1/registration_tokens/{t}",
@@ -1213,8 +1247,8 @@ class SynapseAdmin(ApiRequest):
                 is valid.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
 
         """
         data = {
@@ -1251,8 +1285,8 @@ class SynapseAdmin(ApiRequest):
                 is valid.
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
 
         """
         # If uses_allowed or expiry time were not provided by the user,
@@ -1287,8 +1321,8 @@ class SynapseAdmin(ApiRequest):
             token (string): The registration token to delete
 
         Returns:
-            string: JSON string containing the admin API's response or None if
-                an exception occured. See Synapse admin API docs for details.
+            string: JSON string containing the Admin API's response or None if
+                an exception occured. See Synapse Admin API docs for details.
 
         """
         # t because query also accepts token when we want it for the
@@ -1328,8 +1362,8 @@ class SynapseAdmin(ApiRequest):
 
         Returns:
             list: A list of dictionaries, each containing the response of
-                what a single notice admin API call returned. Usually that is
-                an event ID or an error. See Synapse admin API docs for
+                what a single notice Admin API call returned. Usually that is
+                an event ID or an error. See Synapse Admin API docs for
                 details.
         """
         data = {
@@ -1366,3 +1400,17 @@ class SynapseAdmin(ApiRequest):
         else:
             data["user_id"] = receivers
             return [self.query("post", "v1/send_server_notice", data=data)]
+
+    def raw_request(self, endpoint, method, data):
+        data_dict = {}
+        if method != "get":
+            self.log.debug("The data we are trying to parse and submit:")
+            self.log.debug(data)
+            try:  # user provided json might be crap
+                data_dict = json.loads(data)
+            except Exception as error:
+                self.log.error("loading data: %s: %s",
+                               type(error).__name__, error)
+                return None
+
+        return self.query(method, endpoint, data=data_dict)
