@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # synadm
-# Copyright (C) 2020-2022 Johannes Tiefenbacher
+# Copyright (C) 2020-2023 Johannes Tiefenbacher
 #
 # synadm is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ from synadm import cli
 
 @cli.root.group()
 def group():
-    """ Manage groups (communities)
+    """ Manage groups (communities).
     """
 
 
@@ -33,10 +33,10 @@ def group():
 @click.argument("group_id", type=str)
 @click.pass_obj
 def delete(helper, group_id):
-    """ Delete a local group (community)
+    """ Delete a local group (community).
     """
     sure = (
-        helper.batch or
+        helper.no_confirm or
         click.prompt("Are you sure you want to delete this group? (y/N)",
                      type=bool, default=False, show_default=False)
     )
